from typing import Dict


class Cache:
    def __init__(self):
        # ordered like:
        # guild_id: data
        self.guilds = {}
        # ordered like:
        # guild_id: [members ordered alphabetically]
        self.members = {}
        # ordered like:
        # guild_id: [channels]
        self.channels = {}
        # ordered like:
        # channel_id: {message_id: data}
        self.messages: Dict[int, Dict] = {}
        # ordered like:
        # user_id: data
        self.presences = {}



from typing import TYPE_CHECKING, List
from datetime import datetime

from .user import User
from .guild import Guild
from .channel import TextChannel

if TYPE_CHECKING:
    from .client import Client

class Message:
    def __init__(self, client: "Client", **data):
        self._client = client
        self.id: int = data['id']
        self.guild = Guild(client, **data['guild'])
        self.author = User(client, **data['author'])
        self.channel = TextChannel(client, **data['channel'])
        self.content: str = data['content']
        self.created_at: datetime = datetime.fromisoformat(data['created_at'])
        self.last_edited: datetime = datetime.fromisoformat(data['last_edited'])
        self.tts: bool = data['tts']
        self.mentions_everyone: bool = data['mentions_everyone']
        self.mentioned_users: List[int] = data['mentioned_users']
        self.pinned: bool = data['pinned']
        self.replied_to: int = data['referenced_message_id']

    async def send(self, content: str, **kwargs):
        return await self.channel.send(content, **kwargs)

    async def reply(self, content: str, **kwargs):
        return await self.channel.send(content, replied_message=self.id, **kwargs)



from typing import TYPE_CHECKING, List, Optional

from .flags import GuildPermissions
from .role import Role

if TYPE_CHECKING:
    from .client import Client

class Guild:
    def __init__(self, client: "Client", **data):
        self._client = client
        self.id = int(data['id'])
        self.name: str = data['name']
        self.description: str = data['description']
        self.vanity_url: str = data['vanity_url']
        self.owner_id = int(data['owner_id'])
        self.nsfw: bool = data['nsfw']
        self.large: bool = data['large']
        self.perferred_locale: str = data['perferred_locale']
        self.permissions = GuildPermissions(int(data['permissions']))
        self.features: List[str] = data['features']
        self.verified: bool = data['verified']

    async def edit(
        self,
        name: Optional[str] = None,
        description: Optional[str] = None,
        nsfw: Optional[bool] = None
    ):
        r = await self._client._http.edit_guild(
            guild_id=self.id, name=name, description=description, nsfw=nsfw
        )


    async def get_roles(self) -> List[Role]:
        roles = []
        rs = await self._client._http.get_roles(self.id)
        for role in rs:
            roles.append(Role(role))
        return roles



import traceback
import multidict

from .http import HTTPClient
from .gateway import Gateway
from .cache import Cache
from .user import ClientUser

class Client:
    def __init__(self, intents: int = 0):
        self.intents = intents
        self._events = multidict.MultiDict()
        self._cache = Cache()

    async def from_token(self, token: str, ws: bool = False):
        self._http = HTTPClient(token)

        if ws:
            self._gateway = Gateway(token=token, client=self)
            await self._gateway.start()
        else:
            self.user = ClientUser(self, **await self._http.get_me())

    @classmethod
    async def signup(
        cls,
        email: str,
        password: str,
        username: str,
        intents: int,
        ws: bool,
        pronouns: str = None,
        bio: str = None,
        locale: str = None,
    ):
        http, _ = HTTPClient.signup(
            email=email,
            password=password,
            username=username,
            pronouns=pronouns,
            bio=bio,
            locale=locale
        )

        ret = cls(intents)
        ret._http = http

        if ws:
            ret._gateway = Gateway(token=http._token, intents=intents, client=ret)
            await ret._gateway.start()

    async def _dispatch(self, event: str, *args):
        callbacks = self._events.getall(event)

        for callback in callbacks:
            try:
                await callback(*args)
            except:
                traceback.print_exc()

    def listen(self, event: str):
        def inner(func):
            self._events.add(event, func)
            return func
        return inner



from typing import TYPE_CHECKING

from .flags import GuildPermissions

if TYPE_CHECKING:
    from .client import Client

class Role:
    def __init__(self, client: "Client", **data):
        self.id: int = int(data['id'])
        self.name: str = data['name']
        self.color: int = data['color']
        self.hoist: bool = data['hoist']
        self.position: int = data['position']
        self.permissions = GuildPermissions(int(data['permissions']))
        self.mentionable: bool = data['mentionable']
        self._client = client

    async def edit(self):
        await self._client._http.edit_role()



import functools

def _has_flag(v: int, f: int) -> bool:
    return True if v & f else False

class UserFlags:
    def __init__(self, __v: int):
        has = functools.partial(_has_flag, __v)

        self.early_supporter = has(1 << 0)
        self.staff = has(1 << 1)
        self.verified = has(1 << 2)
        self.likely_scammer = has(1 << 3)
        self.donator = has(1 << 4)
        self.contributor = has(1 << 5)
        self.bug_hunter = has(1 << 6)

class GuildPermissions:
    def __init__(self, __v: int):
        has = functools.partial(_has_flag, __v)

        self.create_invites = has(1 << 0)
        self.kick_members = has(1 << 1)
        self.ban_members = has(1 << 2)
        self.administator = has(1 << 3)
        self.manage_channels = has(1 << 4)
        self.manage_guild = has(1 << 5)
        self.add_reactions = has(1 << 6)
        self.view_audit_log = has(1 << 7)

        # self.priority_speaker = has(1 << 8)
        # self.stream = has(1 << 9)
        self.view_channels = has(1 << 10)
        self.send_messages = has(1 << 11)
        self.send_tts_messages = has(1 << 12)
        self.manage_messages = has(1 << 13)
        self.embed_links = has(1 << 14)
        self.attach_files = has(1 << 15)
        self.read_message_history = has(1 << 16)
        self.mention_everyone = has(1 << 17)
        self.use_external_emojis = has(1 << 18)

        # self.view_guild_insights = has(1 << 19)

        # self.connect = has(1 << 20)
        # self.speak = has(1 << 21)
        # self.mute_members = has(1 << 22)
        # self.deafen_members = has(1 << 23)
        # self.move_members = has(1 << 24)

        self.change_nick = has(1 << 25)
        self.manage_nicks = has(1 << 26)
        self.manage_roles = has(1 << 27)
        self.manage_webhooks = has(1 << 28)
        self.manage_emojis = has(1 << 29)

        # self.manage_events = has(1 << 30)

        self.manage_channel_pins = has(1 << 31)
        self.create_audits = has(1 << 32)

class PermissionOverwrite:
    def __init__(self, allow: int, deny: int, user_id: int):
        self.allow = GuildPermissions(allow)
        self.deny = GuildPermissions(deny)
        self.user_id = user_id

def from_list(obj):
    ret = []
    for overwrite in obj:
        ret.append(PermissionOverwrite(overwrite['allow'], overwrite['deny'], overwrite['user_id']))

    return ret



from typing import TYPE_CHECKING, List, Optional

from .flags import from_list, PermissionOverwrite
from .guild import Guild

if TYPE_CHECKING:
    from .client import Client

class BaseChannel:
    def __init__(self, client: "Client", **data):
        self._client = client
        self.id = int(data['id'])
        self.type: int = data['type']
        self.position: int = data['position']
        self.name: str = data['name']
        self.topic: str = data['topic']
        self.slowmode_timeout: int = data['slowmode_timeout']
        self.parent_id: int = data['parent_id']
        self.permission_overwrites = from_list(data['permission_overwrites'])
        self.guild = Guild(client, **data['guild'])

    async def edit(
        self,
        name: Optional[str] = None,
        topic: Optional[str] = None,
        position: Optional[int] = None,
        permission_overwrites: Optional[List[PermissionOverwrite]] = None,
        slowmode_timeout: Optional[int] = None,
        parent_id: Optional[int] = None
    ):
        permo = None

        if permission_overwrites:
            permo = []
            for overwrite in permission_overwrites:
                permo.append({'allow': overwrite.allow, 'deny': overwrite.deny, 'user_id': overwrite.user_id})

        await self._client._http.edit_channel(
            self.guild.id,
            self.id,
            name=name,
            topic=topic,
            position=position,
            permission_overwrites=permo,
            slowmode_timeout=slowmode_timeout,
            parent_id=parent_id
        )

        if name:
            self.name = name
        if topic:
            self.topic = topic
        if position:
            self.position = position
        if permission_overwrites:
            self.permission_overwrites = permission_overwrites
        if slowmode_timeout:
            self.slowmode_timeout = slowmode_timeout
        if parent_id:
            self.parent_id = parent_id

    async def delete(self):
        await self._client._http.delete_channel(self.guild.id, self.id)

class Category(BaseChannel):
    ...

class TextChannel(BaseChannel):
    async def send(self, content: str, *, replied_message: Optional[int] = None):
        await self._client._http.create_message(
            self.guild.id, self.id, content, replied_message
        )

    #async def get_message(self, message_id: int):
        #return Message(self._client, **await self._client._http.get_message(self.guild.id, self.id, message_id=message_id))

    #async def get_messages(self):
        #msgs = []

        #resp = await self._client._http.get_messages(self.guild.id, self.id)

        #for message in resp:
            #message['channel'] = self
            #msgs.append(Message(self._client, **message))

        #return msgs



import datetime
from typing import TYPE_CHECKING, Optional

from .flags import UserFlags
from .settings import UserSettings

if TYPE_CHECKING:
    from .client import Client

class User:
    def __init__(
        self,
        client: "Client",
        **data
    ):
        self.id: int = int(data['id'])
        self.username: str = data['username']
        self.discriminator: int = data['discriminator']
        if data.get('email'):
            self.email: str = data['email']
        self.flags = UserFlags(data['flags'])
        self.locale: str = data['locale']
        self.joined_at: datetime.datetime = datetime.datetime.fromisoformat(data['joined_at'])
        self.bio: str = data['bio']
        self.verified: bool = data['verified']
        self.system: bool = data['system']
        self.bot: bool = data['bot']
        self.pronouns: str = data['pronouns']
        self._client = client

    async def create_note(self, content: str):
        if hasattr(self, 'edit'):
            raise Exception('Cannot add a note to yourself')

        await self._client._http.create_user_note(
            self.id, content=content
        )

class ClientUser(User):
    async def edit(
        self,
        username: Optional[str] = None,
        discriminator: Optional[int] = None,
        pronouns: Optional[str] = None,
        email: Optional[str] = None,
        password: Optional[str] = None
    ):
        await self._client._http.edit_me(
            username=username,
            discriminator=discriminator,
            pronouns=pronouns,
            email=email,
            password=password
        )

        if username:
            self.username = username
        if discriminator:
            self.discriminator = discriminator
        if pronouns:
            self.pronouns = pronouns
        if email:
            self.email = email

    async def verify(self, code: int):
        if self.verified:
            raise Exception('Already Verified')

        await self._client._http.verify_email(code)

    @property
    async def settings(self):
        r = await self._client._http.get_settings()

        return UserSettings(
            self._client,
            r['theme'],
            r['guild_placements'],
            r['direct_message_ignored_guilds']
        )



from typing import List, Literal, Union

import aiohttp

class HTTPException(Exception):
    pass

class HTTPClient:
    def __init__(self, token: str):
        self._token = token
        self._headers = {
            'Authorization': token,
            'User-Agent': 'venera.py'
        }
        self._session = None
        self._base_url = 'https://concord.chat/api/v5'

    async def check_session(self):
        if not self._session:
            self._session = aiohttp.ClientSession(headers=self._headers)

    async def request(self, method: str, prefix: str, data: dict = None):
        await self.check_session()

        r = await self._session.request(method, self._base_url + prefix, json=data)

        if not r.ok and r.status != 404:
            raise HTTPException(r.status, await r.text())

        return r

    # START SECTION USERS #

    @classmethod
    async def login(cls, email: str, password: str):
        cs = aiohttp.ClientSession()

        r = await cs.get(
            'https://concord.chat/api/v5/users/@me/tokens',
            json={
                'email': email,
                'password': password
            }
        )

        if r.status != 201:
            raise HTTPException('Failed to Login to Account')

        token = await r.json()[0]
        await cs.close()

        return cls(token)

    @classmethod
    async def signup(
        cls,
        email: str,
        password: str,
        username: str,
        locale: str = None,
        bio: str = None,
        pronouns: str = None
    ):
        cs = aiohttp.ClientSession()
        data = {
            'email': email,
            'password': password,
            'username': username
        }

        if locale:
            data['locale'] = locale

        if bio:
            data['bio'] = bio

        if pronouns:
            data['pronouns'] = pronouns

        r = await cs.post(
            'https://concord.chat/api/v5/users',
            json=data
        )

        if r.status != 201:
            raise HTTPException(r.status, await r.text())

        resp = cls(await r.json()['token'])
        return resp, await r.json()

    async def get_me(self):
        r = await self.request('GET', '/users/@me')
        return await r.json()

    async def get_user(self, user_id: int):
        r = await self.request('GET', f'/users/{user_id}')
        return await r.json()

    async def edit_me(
        self,
        username: str = None,
        discriminator: int = None,
        pronouns: str = None,
        email: str = None,
        password: str = None
    ):
        d = {}

        if username:
            d['username'] = username

        if discriminator:
            d['discriminator'] = discriminator

        if pronouns:
            d['pronouns'] = pronouns

        if email:
            d['email'] = email

        if password:
            d['password'] = email

        r = await self.request('PATCH', '/users/@me', d)
        return await r.json()

    async def verify_email(self, code: int):
        try:
            r = await self.request(
                'POST',
                f'/users/@me/verify?utm_verification={code}'
            )
            return await r.json()
        except:
            raise HTTPException('Invalid Verification Code Given')

    async def get_settings(self):
        r = await self.request('GET', '/user/@me/meta')
        return await r.json()

    async def edit_settings(
        self,
        theme: Union[Literal['dark'], Literal['light']] = None,
        guild_placements: List[int] = None,
        direct_message_ignored_guilds: List[int] = None
    ):
        d = {}

        if theme:
            d['theme'] = theme

        if guild_placements:
            d['guild_placements'] = guild_placements

        if direct_message_ignored_guilds:
            d['direct_message_ignored_guilds'] = direct_message_ignored_guilds

        r = await self.request('PATCH', '/users/@me/meta', d)

    async def get_user_note(self, user_id: int):
        r = await self.request('GET', f'/users/@me/notes/{user_id}')
        if r.status == 404:
            return {'s': 'notfound'}

        return await r.json()

    async def create_user_note(
        self,
        user_id: int,
        content: str
    ):
        r = await self.request(
            'PUT',
            f'/users/@me/notes/{user_id}',
            {'content': content}
        )

        return await r.json()

    # END SECTION USERS #
    # START SECTION GUILDS #

    async def get_joined_guilds(self):
        r = await self.request(
            'GET', '/users/@me/guilds'
        )
        return await r.json()

    async def get_guild_user_settings(self, guild_id: int):
        r = await self.request('GET', f'/users/@me/guilds/{guild_id}/meta')
        return await r.json()

    async def edit_guild_user_settings(self, guild_id: int, muted_channels: List[int] = None):
        d = {}

        if muted_channels:
            d['muted_channels'] = muted_channels

        r = await self.request('PATCH', f'/users/@me/guilds/{guild_id}/meta', d)

        return await r.json()

    async def create_guild(
        self,
        name: str,
        description: str = None,
        nsfw: bool = False
    ):
        d = {
            'name': name,
            'description': description
        }

        if nsfw:
            d['nsfw'] = True

        r = await self.request('POST', '/guilds', d)

        return await r.json()

    async def edit_guild(
        self,
        guild_id: int,
        name: str = None,
        description: str = None,
        nsfw: bool = None
    ):
        d = {}

        if name:
            d['name'] = name

        if description:
            d['description'] = description

        if nsfw is not None:
            d['nsfw'] = nsfw

        r = await self.request('PATCH', f'/guilds/{guild_id}', d)

        return await r.json()

    async def delete_guild(
        self, guild_id: int
    ):
        r = await self.request('DELETE', f'/guilds/{guild_id}')

        return await r.json()

    async def get_guild(self, guild_id: int):
        r = await self.request('GET', f'/guilds/{guild_id}')

        if r.status == 404:
            return {'s': 'notfound'}

        return await r.json()

    async def create_invite(self, guild_id: int, channel_id: int, ttl: int = None):
        if ttl:
            d = {'ttl': ttl}
        else:
            d = None

        r = await self.request(
            'POST',
            f'/guilds/{guild_id}/channels/{channel_id}/invites',
            d    
        )

        return await r.json()

    async def claim_vanity(self, guild_id: int, code: str):
        r = await self.request('PUT', f'/guilds/{guild_id}/vanity?utm_vanity={code}')
        return await r.json()

    # END SECTION GUILDS #
    # START SECTION MEMBERS #

    async def get_member(self, guild_id: int, user_id: int):
        r = await self.request('GET', f'/guilds/{guild_id}/members/{user_id}')

        if r.status == 404:
            return {'s': 'notfound'}

        return await r.json()

    async def get_members(self, guild_id: int):
        r = await self.request('GET', f'/guilds/{guild_id}/members')

        return await r.json()

    async def edit_my_member(self, guild_id: int, nick: str = None):
        d = {}

        if nick:
            d['nick'] = nick

        r = await self.request('PATCH', f'/guilds/{guild_id}/members/@me')

        return await r.json()

    async def edit_member(
        self,
        guild_id: int,
        member_id: int,
        nick: str = None,
        roles: List[int] = None
    ):
        d = {}

        if nick:
            d['nick'] = nick

        if roles:
            d['roles'] = roles

        r = await self.request(
            'PATCH',
            f'/guilds/{guild_id}/members/{member_id}/nick',
            d
        )

        return await r.json()

    # END SECTION MEMBERS #
    # START SECTION AUDITS #

    async def get_guild_logs(self, guild_id: int):
        r = await self.request('GET', f'/guilds/{guild_id}/audits')
        return await r.json()

    async def get_guild_log(self, guild_id: int, log_id: int):
        r = await self.request('GET', f'guilds/{guild_id}/audits/{log_id}')
        return await r.json()

    async def create_log(
        self,
        guild_id: int,
        type: str,
        postmortem: str,
        audited: int = None,
        object_id: int = None
    ):
        d = {
            'type': type,
            'postmortem': postmortem
        }

        if audited:
            d['audited'] = audited

        if object_id:
            d['object_id'] = object_id
    
        r = await self.request('POST', f'/guilds/{guild_id}/audits', d)

        return await r.json()

    # END SECTION AUDITS #
    # START SECTION ROLES #

    async def create_role(self):
        raise NotImplementedError

    async def edit_role(self):
        raise NotImplementedError

    async def get_role(self, guild_id: int, role_id: int):
        r = await self.request('GET', f'/guilds/{guild_id}/roles/{role_id}')
        return await r.json()

    async def get_roles(self, guild_id: int):
        r = await self.request('GET', f'/guilds/{guild_id}/roles')
        return await r.json()

    # END SECTION ROLES #
    # START SECTION CHANNELS #

    async def create_channel(
        self,
        guild_id: int,
        type: int,
        position: int,
        parent_id: int = None,
        slowmode_timeout: int = None,
    ):
        d = {
            'type': type,
            'position': position,
        }
    
        if parent_id:
            d['parent_id'] = parent_id

        if slowmode_timeout:
            d['slowmode_timeout'] = slowmode_timeout
    
        r = await self.request(
            'POST', f'/guilds/{guild_id}/channels'
        )

        return await r.json()

    async def edit_channel(
        self,
        guild_id: int,
        channel_id: int,
        name: str = None,
        topic: str = None,
        position: int = None,
        permission_overwrites: List[dict] = None,
        slowmode_timeout: int = None,
        parent_id: int = None
    ):
        d = {}

        if name:
            d['name'] = name

        if topic:
            d['topic'] = topic

        if position:
            d['position'] = position
    
        if permission_overwrites:
            d['permission_overwrites'] = permission_overwrites

        if slowmode_timeout:
            d['slowmode_timeout'] = slowmode_timeout

        if parent_id:
            d['parent_id'] = parent_id

        r = await self.request(
            'PATCH', f'/guilds/{guild_id}/channels/{channel_id}', d
        )

        return await r.json()

    async def delete_channel(self, guild_id: int, channel_id: int):
        r = await self.request('DELETE', f'/guilds/{guild_id}/channels/{channel_id}')
        return await r.json()

    async def get_channel(self, guild_id: int, channel_id: int):
        r = await self.request('GET', f'/guilds/{guild_id}/channels/{channel_id}')
        return await r.json()

    async def get_channels(self, guild_id: int):
        r = await self.request('GET', f'/guilds/{guild_id}/channels')
        return await r.json()

    # END SECTION CHANNELS #
    # START SECTION READSTATES #

    async def ack_message(self, guild_id: int, channel_id: int, message_id: int):
        r = await self.request(
            'POST',
            f'/guilds/{guild_id}/channels/{channel_id}/messages/{message_id}/ack'
        )
        return await r.json()

    async def get_readstate(self, guild_id: int, channel_id: int):
        r = await self.request(
            'GET', f'/guilds/{guild_id}/channels/{channel_id}/readstate'
        )
        return await r.json()

    async def get_readstates(self):
        r = await self.request('GET', '/readstates')
        return await r.json()

    # END SECTION READSTATES #
    # START SECTION MESSAGES #

    async def get_message(self, guild_id: int, channel_id: int, message_id: int):
        r = await self.request(
            'GET',
            f'/guilds/{guild_id}/channels/{channel_id}/messages/{message_id}'
        )
        return await r.json()

    async def get_messages(self, guild_id: int, channel_id: int):
        r = await self.request(
            'GET',
            f'/guilds/{guild_id}/channels/{channel_id}/messages'
        )
        return await r.json()

    async def create_message(
        self,
        guild_id: int,
        channel_id: int,
        content: str,
        referenced_message_id: int = None
    ):
        d = {
            'content': content
        }

        if referenced_message_id:
            d['referenced_message_id'] = referenced_message_id

        r = await self.request(
            'POST', f'/guilds/{guild_id}/channels/{channel_id}/messages', d
        )

        return await r.json()

    async def edit_message(
        self,
        guild_id: int,
        channel_id: int,
        message_id: int,
        content: str = None
    ):
        d = {}

        if content:
            d['content'] = content

        r = await self.request(
            'PATCH', f'/guilds/{guild_id}/channels/{channel_id}/messages/{message_id}'
        )
        return await r.json()

    async def delete_message(self, guild_id: int, channel_id: int, message_id: int):
        r = await self.request('DELETE', f'/guilds/{guild_id}/channels/{channel_id}/messages/{message_id}')
        return await r.json()

    async def pin(self, guild_id: int, channel_id: int, message_id: int):
        r = await self.request('POST', f'/guilds/{guild_id}/channels/{channel_id}/pins/{message_id}')
        return await r.json()

    async def unpin(self, guild_id: int, channel_id: int, message_id: int):
        r = await self.request('DELETE', f'/guilds/{guild_id}/channel/{channel_id}/pins/{message_id}')
        return await r.json()

    # END SECTION MESSAGES #



import setuptools

__version__ = str('0.1.0')

requirements = []
with open('requirements.txt') as f:
    requirements = f.read().splitlines()

packages = [
    'venera'
]


setuptools.setup(
    name='venera.py',
    version=__version__,
    packages=packages,
    url='https://github.com/veneralab/venera.py',
    license='MIT',
    author='Venera',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    install_requires=requirements,
    description='Asynchronous Venera API Wrapper',
    python_requires='>=3.9',
)



import asyncio
import json

import aiohttp
from typing import TYPE_CHECKING

from .message import Message
from .channel import BaseChannel, TextChannel, Category

if TYPE_CHECKING:
    from .client import Client

class Gateway:
    def __init__(self, token: str, intents: int, client: "Client"):
        self._token = token
        self._client = client
        self._intents = intents

    async def start(self):
        self._session = aiohttp.ClientSession()

        self._ws = await self._session.ws_connect('wss://gateway.concord.chat')

        await self._ws.send_str(json.dumps({
            'token': self._token,
            'intents': self._intents
        }))
        loop = asyncio.get_running_loop()
        loop.create_task(self.recv())

    async def recv(self):
        async for msg in self._ws:
            if msg.type == aiohttp.WSMsgType.TEXT:
                data: dict = json.loads(msg.data)
                if data.get('t'):
                    await self.handle_event(data['t'], data)
            if msg.type == aiohttp.WSMsgType.CLOSE:
                # TODO: Handle
                ...

    async def handle_event(self, t: str, data: dict):
        d: dict = data['d']

        if t.startswith('MESSAGE'):
            channel = self._client._cache.messages.get(int(d['channel']['id']))

            if channel is None:
                channel = self._client._cache.messages[int(d['channel']['id'])] = {}

            if d['channel']['type'] == 1:
                d['channel'] = TextChannel(self._client, **d['channel'])

        if t == 'MESSAGE_CREATE':
            msg = Message(self._client, **d)
            channel[msg.id] = {int(d['id']): msg}
            await self._client._dispatch('MESSAGE_CREATE', msg)

        elif t == 'MESSAGE_EDIT':
            channel[int(d['id'])] = Message(self._client, **d)
            await self._client._dispatch('MESSAGE_EDIT', channel[int(d['id'])])

        elif t == 'MESSAGE_DELETE':
            channel.pop(int(d['id']))
            await self._client._dispatch('MESSAGE_DELETE')

        elif t == 'CHANNEL_CREATE':
            if d['type'] == 0:
                channel = Category(self._client, **d)
            if d['type'] == 1:
                channel = TextChannel(self._client, **d)
            else:
                channel = BaseChannel(self._client, **d)

            self._client._cache.channels[channel.id] = channel



from typing import TYPE_CHECKING, List, Literal, Union

if TYPE_CHECKING:
    from .client import Client

class UserSettings:
    def __init__(
        self,
        client: "Client",
        theme: str,
        guild_placements: List[int],
        direct_message_ignored_guilds: List[int]
    ):
        self.theme = theme
        self.guild_placements = guild_placements
        self.direct_message_ignored_guilds = direct_message_ignored_guilds
        self._client = client

    async def edit(
        self,
        theme: Union[
            Literal['dark'],
            Literal['light']
        ] = None,
        guild_placements: List[int] = None,
        direct_message_ignored_guilds: List[int] = None
    ):
        await self._client._http.edit_settings(
            theme=theme,
            guild_placements=guild_placements,
            direct_message_ignored_guilds=direct_message_ignored_guilds
        )

        if theme:
            self.theme = theme
        if guild_placements:
            self.guild_placements = guild_placements
        if direct_message_ignored_guilds:
            self.direct_message_ignored_guilds = direct_message_ignored_guilds

class GuildSettings:
    def __init__(self, client: "Client", guild_id: int, muted_channels: List[int] = None):
        self._client = client
        self.guild_id = guild_id
        self.muted_channels = muted_channels
    
    async def edit(self, muted_channels: List[int] = None):
        await self._client._http.edit_guild_user_settings(
            guild_id=self.guild_id,
            muted_channels=muted_channels
        )

        if muted_channels:
            self.muted_channels = muted_channels
